import osw.model.entity as model
from osw.utils.strings import pascal_case
from osw.utils.wiki import get_namespace


def test_get_entity_namespace():
    class DummyClass(model.Entity):
        pass

    assert get_namespace(DummyClass) == "Category"

    e = model.Entity(label=[model.Label(text="MyItem")])
    assert get_namespace(e) is None

    # e = model.Item(label=[model.Label(text="MyItem")])
    # assert get_namespace(e) == "Item"

    # not yet loaded on default
    # e = model.Category(label=[model.Label(text="MyItem")])
    # assert get_namespace(e) == "Category"

    # e = model.Property(label=[model.Label(text="MyItem")])
    # assert get_namespace(e) == "Property"

    # e = model.WikiFile(label=[model.Label(text="MyItem")])
    # assert get_namespace(e) == "File"


def test_strings_pascalcase():
    assert pascal_case("my string") == "MyString"
    assert pascal_case("my_string") == "MyString"
    assert pascal_case("MyString") == "MyString"



import os
import shutil
from pathlib import Path
from typing import IO, Optional

from osw.controller.file.base import FileController
from osw.core import model


class LocalFileController(FileController, model.LocalFile):
    label: Optional[model.Label] = model.Label(text="Unnamed file")
    path: Path

    class Config:
        arbitrary_types_allowed = True

    def get(self) -> IO:
        self._set_metadata()
        return open(self.path, "rb")

    def put(self, file: IO):
        self._set_metadata()
        with open(self.path, "wb") as f:
            # f.write(file.read()) # in-memory - limited by available RAM
            shutil.copyfileobj(file, f)  # chunked copy

    def delete(self):
        if os.path.exists(self.path):
            os.remove(self.path)

    def _set_metadata(self):
        file = LocalFileController.extract_metadata(self.path)
        self.label = file.label

    @classmethod
    def extract_metadata(cls, path: Path) -> model.LocalFile:
        file = model.LocalFile(label=[model.Label(text=path.name)])
        if os.path.exists(path):
            pass
            # stats = os.stat(path)
            # TODO: extract metadata
            # ... file.size = stats.st_size
        return file



import os
from importlib import reload
from pprint import pprint
from typing import ClassVar
from uuid import uuid4

from pydantic import BaseModel

import osw.model.entity as model
from osw.core import OSW
from osw.wtsite import WtSite

# create/update the password file under examples/accounts.pwd.yaml
pwd_file_path = os.path.join(
    os.path.dirname(os.path.abspath(__file__)), "accounts.pwd.yaml"
)
wtsite = WtSite.from_domain("wiki-dev.open-semantic-lab.org", pwd_file_path)
osw = OSW(site=wtsite)


# todo: does dataclass export only most specific class jsonschema
class MyPythonClass(
    BaseModel
):  # we don't inherit from model.Item here because this will trigger a full schema export
    __uuid__: ClassVar[uuid4] = "23e4356e-b726-4c5b-b63f-620b301eb836"
    my_value: str


osw.register_schema(
    osw.SchemaRegistration(
        model_cls=MyPythonClass,
        schema_uuid=MyPythonClass.__uuid__,
        schema_name="MyPythonClass",
        schema_bases=["Category:Item"],
    )
)

osw.fetch_schema(
    osw.FetchSchemaParam(
        schema_title="Category:" + OSW.get_osw_id(MyPythonClass.__uuid__), mode="append"
    )
)
reload(model)

my_instance = model.MyPythonClass(
    label=[model.Label(text="My Instance")], my_value="test"
)
pprint(my_instance)



# from quantities
import getpass

from SPARQLWrapper import BASIC, JSON, POST, SPARQLWrapper


class SmwSparqlClient:
    def __init__(self, endpoint, domain, auth="none", user="", password=""):
        self.create_sparql_client(endpoint, auth, user, password)

        self.prefix_dict = {
            "dc": "http://purl.org/dc/elements/1.1/",
            "property": "https://{domain}/id/Property-3A",
            "file": "https://{domain}/id/File-3A",
            "category": "https://{domain}/id/Category-3A",
            "material": "https://{domain}/id/Material-3A",
            "wiki": "https://{domain}/id/",
            "rdf": "http://www.w3.org/1999/02/22-rdf-syntax-ns#",
            "rdfs": "http://www.w3.org/2000/01/rdf-schema#",
            "owl": "http://www.w3.org/2002/07/owl#",
            "swivt": "http://semantic-mediawiki.org/swivt/1.0#",
            "xsd": "http://www.w3.org/2001/XMLSchema#",
        }
        self.wiki_prefixes = []
        self.prefixes = ""
        for prefix in self.prefix_dict:
            if "{domain}" in prefix:
                self.wiki_prefixes.append[prefix]
            self.prefix_dict[prefix] = self.prefix_dict[prefix].replace(
                "{domain}", domain
            )
            self.prefixes += f"PREFIX {prefix}: <{self.prefix_dict[prefix]}>\n".replace(
                "{domain}", domain
            )

        # from https://github.com/SemanticMediaWiki/SemanticMediaWiki/blob/e04b78ddbc0a1b2181b12f31a51d7f91e723336b/src/Exporter/Escaper.php#L61
        self.smw_encoding_lists = {
            "in": [
                "*",
                ",",
                ";",
                "<",
                ">",
                "(",
                ")",
                "[",
                "]",
                "{",
                "}",
                "\\",
                "$",
                "^",
                ":",
                '"',
                "#",
                "&",
                "'",
                "+",
                "!",
                "%",
            ],
            "out": [
                "-2A",
                "-2C",
                "-3B",
                "-3C",
                "-3E",
                "-28",
                "-29",
                "-5B",
                "-5D",
                "-7B",
                "-7D",
                "-5C",
                "-24",
                "-5E",
                "-3A",
                "-22",
                "-23",
                "-26",
                "-27",
                "-2B",
                "-21",
                "-",
            ],
        }
        self.encoding_dict = dict(
            zip(self.smw_encoding_lists["in"], self.smw_encoding_lists["out"])
        )
        self.encoding_dict = {}
        self.encoding_dict_inv = dict(
            (self.encoding_dict[k], k) for k in self.encoding_dict
        )

    def create_sparql_client(self, endpoint, auth="none", user="", password=""):
        self.sparql = SPARQLWrapper(endpoint)
        if auth == "basic":
            self.sparql.setHTTPAuth(BASIC)
            if user == "":
                user = input("Enter username")
            if password == "":
                password = getpass.getpass("Enter password")
            self.sparql.setCredentials(user, password)
        self.sparql.setMethod(POST)

    def sparqlQuery(self, query):
        # print(query)
        self.sparql.setQuery(query)
        self.sparql.setReturnFormat(JSON)
        # print(self.sparql.query())
        return self.sparql.query().convert()

    def get_sparql_triplets(
        self,
        where_statement,
        select_statement="",
        inner_where_statement="?subject ?predicate ?object",
        tsubject="",
        tpredicate="",
        tobject="",
        limit=999999999,
        prefixes="",
        debug=False,
    ):
        select_statement = ""
        if tsubject == "":
            select_statement += " ?subject"
        if tpredicate == "":
            select_statement += " ?predicate"
        if tobject == "":
            select_statement += " ?object"
        if tsubject != "":
            inner_where_statement = inner_where_statement.replace(
                "?subject", "<" + tsubject + ">"
            )
        if tpredicate != "":
            inner_where_statement = inner_where_statement.replace(
                "?predicate", "<" + tpredicate + ">"
            )
        if tobject != "":
            inner_where_statement = inner_where_statement.replace(
                "?object", "<" + tobject + ">"
            )
        if prefixes == "":
            prefixes = self.prefixes
        query = (
            """
            {{{prefixes}}}
            SELECT {{{select_statement}}}
            WHERE {
              {{{where_statement}}}
                {
                SELECT {{{select_statement}}}
                WHERE {
                    {{{inner_where_statement}}}
                }
              }
            }
            LIMIT {{{limit}}}
        """.replace(
                "{{{select_statement}}}", select_statement
            )
            .replace("{{{where_statement}}}", where_statement)
            .replace("{{{inner_where_statement}}}", inner_where_statement)
            .replace("{{{limit}}}", str(limit))
            .replace("{{{prefixes}}}", prefixes)
        )
        for key in self.encoding_dict.keys():
            query = query.replace(key, self.encoding_dict[key])
        if debug:
            print(query)
        # select distinct ?p ?o where {<http://dbpedia.org/resource/Amount_of_substance> ?p ?o} LIMIT 100
        triples = self.sparqlQuery(query)
        # print(triples)
        tdict = {}
        if debug:
            print("{} triplets found".format(len(triples["results"]["bindings"])))
        for t in triples["results"]["bindings"]:
            if tsubject == "":
                s = t["subject"]["value"]
            else:
                s = tsubject
            if tpredicate == "":
                p = t["predicate"]["value"]
            else:
                p = tpredicate
            if tobject == "":
                o = t["object"]["value"]
            else:
                o = tobject
            if "xml:lang" in t["object"]:
                o += "@" + t["object"]["xml:lang"]
            # print("{} {} {}".format(s,p,o))
            for prefix in self.prefix_dict:
                s = s.replace(self.prefix_dict[prefix], prefix + ":")
                p = p.replace(self.prefix_dict[prefix], prefix + ":")
                o = o.replace(self.prefix_dict[prefix], prefix + ":")
            for key in self.encoding_dict_inv.keys():
                s = s.replace(key, self.encoding_dict_inv[key])
                p = p.replace(key, self.encoding_dict_inv[key])
                o = o.replace(key, self.encoding_dict_inv[key])
            self.dict_append_tripl(tdict, s, p, o)
        # print(tdict)
        return tdict

    def spo_result_to_dict(self, triples, tsubject="", tpredicate="", tobject=""):
        tdict = {}
        for t in triples["results"]["bindings"]:
            if tsubject == "":
                s = t["subject"]["value"]
            else:
                s = tsubject
            if tpredicate == "":
                p = t["predicate"]["value"]
            else:
                p = tpredicate
            if tobject == "":
                o = t["object"]["value"]
            else:
                o = tobject
            if "xml:lang" in t["object"]:
                o += "@" + t["object"]["xml:lang"]
            # print("{} {} {}".format(s,p,o))
            for prefix in self.prefix_dict:
                s = s.replace(self.prefix_dict[prefix], prefix + ":")
                p = p.replace(self.prefix_dict[prefix], prefix + ":")
                o = o.replace(self.prefix_dict[prefix], prefix + ":")
            self.dict_append_tripl(tdict, s, p, o)
        # print(tdict)
        return tdict

    def dict_append_tripl(self, d, s, p, o):
        if s not in d:
            d[s] = {}
        if p not in d[s]:
            d[s][p] = []
        d[s][p].append(o)



from abc import abstractmethod
from typing import IO

from osw.controller.file.base import FileController
from osw.core import model

# TODO: add addional remove file with https://docs.prefect.io/2.11.4/concepts/filesystems/


# note: the order of the base classes is important
# the data class must be the first base class, otherwise subclass controllers fall back to the data model of the controller superclass
class RemoteFileController(model.RemoteFile, FileController):
    @abstractmethod
    def get() -> IO:
        pass

    @abstractmethod
    def put(file: IO):
        pass

    # @classmethod
    # def from_local(self, local: "local.LocalFileController") -> "RemoteFileController":
    #    rf = local.cast(self)
    #    rf.put(local.get())
    #    return rf

    # def from_local(self, local: "local.LocalFileController") -> None:
    #    with local.get() as f:
    #        self.put(f)



import osw.wiki_tools as wt


def test_create_flat_content_structure_from_wikitext():
    wikitext = "{{Template" "|param=value" "}}"
    expected = [{"Template": {"param": ["value"]}}]
    result = wt.create_flat_content_structure_from_wikitext(wikitext)
    assert result == expected



from typing import Type

from pydantic import create_model

import osw.model.entity as model


class ControllerTestController(model.Entity):
    def testfunction(self):
        print(f"Entity with label '{str(self.label.text)}'")


def controller_mixin(cls) -> Type:
    # import osw.controller.Entity as controller
    prefix = ""  # "controller."
    postfix = "Controller"
    try:
        eval(f"{prefix}{cls.__name__}{postfix}")
    except NameError:
        # print(f"controller.{cls.__name__} No defined")
        return cls
    else:
        # the controller class
        ctrl_cls = eval(f"{prefix}{cls.__name__}{postfix}")
        # the patched class inheriting both from the data and the controller class
        mixin_cls = create_model(cls.__name__, __base__=tuple([ctrl_cls, cls]))
        return mixin_cls


class ControllerTest(model.Entity):
    pass


ControllerTest = controller_mixin(ControllerTest)


class ControllerTestSubclass(ControllerTest):
    pass


ControllerTestSubclass = controller_mixin(ControllerTestSubclass)

test = ControllerTestSubclass(label=model.Label(text="MyTest"))
test.testfunction()  # while this works, theres no support from static type checking



# This file is required to make Python treat directories containing the file as packages.
# Additionally, classes from multiple files are imported to allow easy access via 'osw.controller.<ClassName>

import inspect
from pprint import pprint

import osw.model.entity as model

# imports of controllers for always present models
from osw.controller.entity import Entity

# conditional imports
models = [m[0] for m in inspect.getmembers(model)]

if "Database" in models:
    from osw.controller.database import DatabaseController



# generated by datamodel-codegen:
#   filename:  Entity.json
#   timestamp: 2023-07-22T07:29:50+00:00

from __future__ import annotations

from enum import Enum
from typing import List, Optional, Union
from uuid import UUID

from pydantic import Field

from osw.model.static import OswBaseModel


class LangCode(Enum):
    en = "en"
    de = "de"


from typing import Type, TypeVar
from uuid import uuid4

from osw.model.static import Ontology, OswBaseModel


class Label(OswBaseModel):
    text: str = Field(..., title="Text")
    lang: Optional[LangCode] = Field("en", title="Lang code")


class Description(OswBaseModel):
    text: str = Field(..., title="Text")
    lang: Optional[LangCode] = Field("en", title="Lang code")


class Statement(OswBaseModel):
    uuid: UUID = Field(default_factory=uuid4, title="UUID")
    label: Optional[List[Label]] = Field(None, title="Label")
    """
    Human readable name
    """
    subject: Optional[str] = Field(None, title="Subject")
    substatements: Optional[
        List[Union[ObjectStatement, DataStatement, QuantityStatement]]
    ] = Field(None, title="Substatements")


class ObjectStatement(Statement):
    predicate: str = Field(..., title="Predicate")
    object: str = Field(..., title="Object")


class DataStatement(Statement):
    property: str = Field(..., title="Property")
    value: str = Field(..., title="Value")


class QuantityStatement(Statement):
    quantity: str = Field(..., title="Property")
    numerical_value: str = Field(..., title="Value")
    unit: str = Field(..., title="Unit")
    unit_symbol: str
    value: str = Field(..., title="Value")


class Entity(OswBaseModel):
    uuid: UUID = Field(default_factory=uuid4, title="UUID")
    name: Optional[str] = Field(None, title="Name")
    """
    Technical / Machine compatible name
    """
    label: List[Label] = Field(..., min_items=1, title="Labels")
    """
    Human readable names. You have to assign at least one.
    """
    short_name: Optional[List[Label]] = Field(None, title="Short name")
    """
    Abbreviation, Acronym, etc.
    """
    query_label: Optional[str] = Field(None, title="Label")
    description: Optional[List[Description]] = Field(None, title="Description")
    image: Optional[str] = Field(None, title="Image")
    based_on: Optional[List[str]] = Field(None, title="Based on")
    """
    Other entities on which this one is based, e.g. when it is created by copying
    """
    statements: Optional[
        List[Union[ObjectStatement, DataStatement, QuantityStatement]]
    ] = Field(None, title="Statements")
    attachments: Optional[List[str]] = Field(None, title="File attachments")


Entity.update_forward_refs()
Statement.update_forward_refs()
ObjectStatement.update_forward_refs()
DataStatement.update_forward_refs()
QuantityStatement.update_forward_refs()
# generated by datamodel-codegen:
#   filename:  Item.json
#   timestamp: 2023-07-22T07:29:52+00:00


from enum import Enum
from typing import List, Optional, Union
from uuid import UUID

from pydantic import Field

from osw.model.static import OswBaseModel


class Level(Enum):
    public = "public"
    internal = "internal"
    restricted = "restricted"


class ReadAccess(OswBaseModel):
    level: Optional[Level] = Field(None, title="Level")


class AccessRestrictions(OswBaseModel):
    read: Optional[ReadAccess] = Field(None, title="Read access")


class Item(Entity):
    type: Optional[List[str]] = Field(
        ["Category:Item"], min_items=1, title="Types/Categories"
    )
    entry_access: Optional[AccessRestrictions] = Field(
        None, title="Access restrictions"
    )


Entity.update_forward_refs()
Statement.update_forward_refs()
ObjectStatement.update_forward_refs()
DataStatement.update_forward_refs()
QuantityStatement.update_forward_refs()
Item.update_forward_refs()
